/*
 * freescope - Free source browser
 * Copyright (C) 2001  Olivier Deme
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/*
 * FILE:        fconfig.h
 *
 * DESCRIPTION: This file defines the class FConfig that contains all 
 *              configuration parameters for freescope.
 */

#ifndef _FCONFIG_H
#define _FCONFIG_H

/************/
/* INCLUDES */
/************/

#include <string>


/*********************/
/* NAMESPACE SYMBOLS */
/*********************/

using std::string;


/**********/
/* MACROS */
/**********/

#define FCONFIG (FConfig::instance())

/**************************/
/* CLASS PRE-DECLARATIONS */
/**************************/


/*********************/
/* CLASS DEFINITIONS */
/*********************/

class FConfig
{
    // TYPE DEFINITIONS
    typedef enum
    {
        KEY_NEXT_SELECTION,
        KEY_PREV_SELECTION,
        KEY_NEXT_PAGE,
        KEY_PREV_PAGE,
        KEY_NEXT_QUERY,
        KEY_PREV_QUERY,
        KEY_SWITCH_MODE,
        KEY_NEXT_COMPLETION,
        KEY_PREV_COMPLETION,
        KEY_REBUILD_DB,
        KEY_SHOW_HELP,
        KEY_ESCAPE,
        KEY_HISTORY_FORWARD,
        KEY_HISTORY_BACKWARD
    } KEY_TYPE;

    // METHODS
public:
    static  FConfig*     instance                 ();
            void         init_config              (const string& config_file);
    static  void         set_db_file              (char* db_file);
            string       get_db_file              ();
    static  void         set_input_file           (char* input_file);
            string       get_input_file           ();
            char*        get_input_file_str       ();
    static  void         set_update_on_startup    (int update);
            int          get_update_on_startup    ();
    static  int          set_key_next_selection   (char key);
            char         get_key_next_selection   ();
    static  int          set_key_prev_selection   (char key);
            char         get_key_prev_selection   ();
    static  int          set_key_next_page        (char key);
            char         get_key_next_page        ();
    static  int          set_key_prev_page        (char key);
            char         get_key_prev_page        ();
    static  int          set_key_next_query       (char key);
            char         get_key_next_query       ();
    static  int          set_key_prev_query       (char key);
            char         get_key_prev_query       ();
    static  int          set_key_switch_mode      (char key);
            char         get_key_switch_mode      ();
    static  int          set_key_next_completion  (char key);
            char         get_key_next_completion  ();
    static  int          set_key_prev_completion  (char key);
            char         get_key_prev_completion  ();
    static  int          set_key_rebuild_db       (char key);
            char         get_key_rebuild_db       ();
    static  int          set_key_help             (char key);
            char         get_key_help             ();
    static  int          set_key_escape           (char key);
            char         get_key_escape           ();
    static  int          set_key_history_forward  (char key);
            char         get_key_history_forward  ();
    static  int          set_key_history_backward (char key);
            char         get_key_history_backward ();
            void         set_dumping_db           ();
            bool         get_dumping_db           ();

private:
    FConfig ();

    bool is_valid_key (char key, KEY_TYPE type);
    
    // ATTRIBUTES
private:
    static FConfig* m_instance;
           string   m_db_file;
           string   m_input_file;
           int      m_update_on_startup;
           char     m_key_next_selection;
           char     m_key_prev_selection;
           char     m_key_next_page;
           char     m_key_prev_page;
           char     m_key_next_query;
           char     m_key_prev_query;
           char     m_key_switch_mode;
           char     m_key_next_completion;
           char     m_key_prev_completion;
           char     m_key_rebuild_db;
           char     m_key_help;
           char     m_key_escape;
           char     m_key_history_forward;
           char     m_key_history_backward;
           bool     m_dumping_db;
};

#endif // _FCONFIG_H
