/*
 * freescope - Free source browser
 * Copyright (C) 2001  Olivier Deme
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

%{


#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "readconfig.h"

extern int g_config_lineno;

SET_DB_FILE             g_set_db_file;
SET_INPUT_FILE          g_set_input_file;
SET_UPDATE_ON_STARTUP   g_set_update_on_startup;
SET_KEY                 g_set_key_next_selection;
SET_KEY                 g_set_key_prev_selection;
SET_KEY                 g_set_key_next_page;
SET_KEY                 g_set_key_prev_page;
SET_KEY                 g_set_key_next_query;
SET_KEY                 g_set_key_prev_query;
SET_KEY                 g_set_key_switch_mode;
SET_KEY                 g_set_key_next_completion;
SET_KEY                 g_set_key_prev_completion;
SET_KEY                 g_set_key_rebuild_db;
SET_KEY                 g_set_key_help;
SET_KEY                 g_set_key_history_forward;
SET_KEY                 g_set_key_history_backward;

static char* file_path;

%}

%union {
    char* string;
    char  character;
}

%token <string> STRING
%token <string> YES
%token <string> NO
%token <character> CHARACTER
%token DB_FILE
%token INPUT_FILE
%token UPDATE_ON_STARTUP
%token KEY_NEXT_SELECTION
%token KEY_PREV_SELECTION
%token KEY_NEXT_PAGE
%token KEY_PREV_PAGE
%token KEY_NEXT_QUERY
%token KEY_PREV_QUERY
%token KEY_SWITCH_MODE
%token KEY_NEXT_COMPLETION
%token KEY_PREV_COMPLETION
%token KEY_REBUILD_DB
%token KEY_HELP
%token KEY_HISTORY_FORWARD
%token KEY_HISTORY_BACKWARD
%token EOL


%%

config_file:    lines
                ;

lines:          one_line lines
                | one_line
                ;

one_line:       LINE_DB_FILE EOL
                | LINE_INPUT_FILE EOL
                | LINE_UPDATE_ON_STARTUP EOL
                | LINE_KEY_NEXT_SELECTION EOL
                | LINE_KEY_PREV_SELECTION EOL
                | LINE_KEY_NEXT_PAGE EOL
                | LINE_KEY_PREV_PAGE EOL
                | LINE_KEY_NEXT_QUERY EOL
                | LINE_KEY_PREV_QUERY EOL
                | LINE_KEY_SWITCH_MODE EOL
                | LINE_KEY_NEXT_COMPLETION EOL
                | LINE_KEY_PREV_COMPLETION EOL
                | LINE_KEY_REBUILD_DB EOL
                | LINE_KEY_HELP EOL
                | LINE_KEY_HISTORY_FORWARD EOL
                | LINE_KEY_HISTORY_BACKWARD EOL
                | EOL
                ;

LINE_DB_FILE:   DB_FILE STRING
                {
                    if (file_path != NULL)
                        free(file_path);
                    
                    file_path = strdup($2);
                    g_set_db_file(file_path);
                }
                ;

LINE_INPUT_FILE:   INPUT_FILE STRING
                {
                    if (file_path != NULL)
                        free(file_path);
                    
                    file_path = strdup($2);
                    g_set_input_file(file_path);
                }
                ;

LINE_UPDATE_ON_STARTUP:	UPDATE_ON_STARTUP YES 
                        {
                            g_set_update_on_startup(1);
                        }
                        | UPDATE_ON_STARTUP NO 
                        { 
                            g_set_update_on_startup(0);
                        }
                        ;

LINE_KEY_NEXT_SELECTION: KEY_NEXT_SELECTION CHARACTER 
                         {
                             if (g_set_key_next_selection($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_PREV_SELECTION: KEY_PREV_SELECTION CHARACTER 
                         {
                             if (g_set_key_prev_selection($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_NEXT_PAGE: KEY_NEXT_PAGE CHARACTER
                         {
                             if (g_set_key_next_page($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_PREV_PAGE: KEY_PREV_PAGE CHARACTER
                         {
                             if (g_set_key_prev_page($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_NEXT_QUERY: KEY_NEXT_QUERY CHARACTER
                         {
                             if (g_set_key_next_query($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_PREV_QUERY: KEY_PREV_QUERY CHARACTER
                         {
                             if (g_set_key_prev_query($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;


LINE_KEY_SWITCH_MODE: KEY_SWITCH_MODE CHARACTER
                         {
                             if (g_set_key_switch_mode($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_NEXT_COMPLETION: KEY_NEXT_COMPLETION CHARACTER
                         {
                             if (g_set_key_next_completion($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_PREV_COMPLETION: KEY_PREV_COMPLETION CHARACTER
                         {
                             if (g_set_key_prev_completion($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_REBUILD_DB: KEY_REBUILD_DB CHARACTER
                         {
                             if (g_set_key_rebuild_db($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_HELP: KEY_HELP CHARACTER
                         {
                             if (g_set_key_help($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_HISTORY_FORWARD: KEY_HISTORY_FORWARD CHARACTER
                         {
                             if (g_set_key_history_forward($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

LINE_KEY_HISTORY_BACKWARD: KEY_HISTORY_BACKWARD CHARACTER
                         {
                             if (g_set_key_history_backward($2) != 0)
                             {
                                 fprintf(stderr,
                                         "Failed to map new key. "
                                         "Conflict? (line %d)\n",
                                         g_config_lineno);
                             }
                         }
                         ;

%%

int yyerror (char* s)
{
    fprintf(stderr, "Error in config file, line %d: %s\n", g_config_lineno, s);
    return 0;
}

void read_config (char* config_file)
{
    FILE* file;

    if ((file = fopen(config_file, "r")) == NULL)
        return;
    
    yyrestart(file);
    yyparse();
    fclose(file);
}
